"""
    Most of the functions in this module are implemented at O(logN) Time Complexity;
    This Module Contains multiple methods to tackle with bit Manipulation problems.
"""

import math as _math


def Binary(value) -> str:
    """
    Args:
        value (int): Enter a value to get binary of that value
    Returns:
        str: returns binary as string
    Complexity: O(Log N)
    """
    return bin(value)[2:];

def Hexa(value) -> str:
    """
    Args:
        value (int): Enter a value to get hexadecimal of that value
    Returns:
        str: returns hexadecimal as string
    Complexity: O(Log N)
    """
    return hex(value)[2:];

def Octal(value) -> str:
    """
    Args:
        value (int): Enter a value to get octadecimal of that value
    Returns:
        str: returns octadecimal as string
    Complexity: O(Log N)
    """
    return oct(value)[2:];

def Toggle(value):
    """
    Args:
        value (int): a number to toggle bits
    Returns:
        _int_: returns the number after toggling it's bits
    Complexity: O(Log N)
    """
    power = 2**(int(_math.log2(value)) + 1) - 1;
    return value ^ power;

def CountSetBits(value) -> int:
    """
    Args:
        value (int): a number to count the set bits

    Returns:
        int: returns set bit count of number
    Complexity: O(Log N)
    """
    count = 0;
    while(value): 
        if value & 1: count += 1;
        value >>= 1;
    return count;
        
def BinToDecimal(binary) -> int:
    """A function to convert binary to decimal

    Args:
        binary (int/string): input binary

    Returns:
        int: returns decimal value of binary
    Complexity: O(Log N)
    """
    binary = str(binary);
    return int(binary, 2);

def OctToDecimal(octal) -> int:
    """A function to convert octal to decimal
    Args:
        octal (int/string): input binary
    Returns:
        int: returns decimal value of octal
    Complexity: O(Log N)
    """
    octal = str(octal);
    return int(octal, 2);

def HexToDecimal(hexadecimal) -> int:
    """A function to convert hexadecimal to decimal
    Args:
        hexadecimal (int/string): input hexadecimal
    Returns:
        int: returns decimal value of hexadecimal
    Complexity: O(Log N)
    """
    hexadecimal = str(hexadecimal);
    return int(hexadecimal, 16);






from collections import Counter as _Counter
from collections import *;
from typing import List as _List

def Length(arr :_List) -> int:
    """
    Args:
        arr (List): input array
    Returns:
        int: returns length of array
        Complexity: O(N)
    """
    return len(arr);

def Unique(arr :_List) -> int:
    """
    Args:
        arr (List): input array
    Returns:
        List: returns a list of unique elements
        Complexity: O(N)
    """
    return list(set(arr))

def Freq(arr :_List) -> dict:
    """
    Args:
        arr (List): input array
    Returns:
        dict: returns a dict containing frequency of each element of array
        Complexity: O(N)
    """
    return dict(_Counter(arr));

def Subseq(arr :_List) -> _List[_List]:
    """A function to generate subsequences of array

    Args:
        arr (List): Input Array

    Returns:
        List[List]: returns a 2D list containing Subsequences
        Complexity: O(2**N)
    """
    def traverse(arr, i, n, seq, ref):
        if (i == n):
            seq.append(ref[:]); return;        
        ref.append(arr[i]);
        traverse(arr, i+1, n, seq, ref);
        ref.pop();
        traverse(arr, i+1, n, seq, ref);       
    seq = []; ref = [];
    traverse(arr, 0, len(arr), seq, ref); del ref;
    return seq;

def Subarr(arr):
    """A functions to get subarrays of arr
    Args:
        arr (List - int/str): input arr to get subarrays
    Returns:
        List[List]: returns an List containing all subarrays
        Complexity: O(N*N)
    """
    substr = []; n = len(arr);
    for i in range(n):
        ref = []
        for j in range(i, n):
            ref.append(arr[j]);
            substr.append(ref[:]);
    del ref;
    return substr;



import bisect as _bisect;

def Find(array, size, value):
    """
    A method to find the index of a value in array at O(LogN) Complexity
    Args:
        array (list): Array to search element
        size (int): Size of Array
        value (int): Value to Search for
    Returns:
        int: Returns index of value if value is found else returns -1
    """
    if (value in array):
        return _bisect.bisect_left(array, value, 0, size);
    return -1;

def UpperBound(array, size, value):
    """
    A method to find the possible index to insert value into the array to maintain Sorted Order
    at O(LogN) Complexity (returns highest possible Index
    Args:
        array (list): Array to search element
        size (int): Size of Array_
        value (int): Value to Search for
    Returns:
        int: Returns index of value if value is found else returns -1
    """
    return _bisect.bisect_right(array, value, 0, size);

def LowerBound(array, size, value):
    """
    A method to find the possible index to insert value into the array to maintain Sorted Order
    at O(LogN) Complexity (returns least possible Index)
    Args:
        array (list): Array to search element
        size (int): Size of Array
        value (int): Value to Search for
    Returns:
        int: Returns index of value if value is found else returns -1
    """
    return _bisect.bisect_left(array, value, 0, size);


    


def Sieve(value):
    """
    A function that generates sieve of erathosthenes till value
       Example: 
                >>> math.sieve(10)
                [0,0,1,1,0,1,0,1,0,0,0] 
                0 -> not prime and 1 -> prime
                
    Args:
        value (int): Number to generate sieve of erathosthenes
    Returns:
        List: returns a list of sieve of erathosthenes
    Complexity: O(N LogN)
    """
    prime = [1]*value;
    prime[0], prime[1] = 0, 0; i = 2;
    while(i*i <= value):
        if (prime[i]):
            for j in range(i*i, value, i):
                if (prime[j]): prime[j] = 0;
        i += 1;
    return prime;

def IsPrime(value):
    """
    Args:
        value (int): value to check whether prime or not
    Returns:
        bool: return True if prime else returns False
    Complexity: O(root N)
    """
    if (value <= 1): return False;
    if (value in (2,3,5,7)): return True;
    if (value <= 10): return False;
    for i in range(2, int(value**0.5)+1):
        if value % i == 0: return False;
    return True;

def Product(arr):
    """ A funtion that returns product of all elements of array
    Args:
        arr (List): Input array
    Returns:
        int: returns product of all elements of array, 0 if array is empty
    Complexity: O(N)
    """
    if (arr):
        i = 1; 
        for val in arr: i *= val;
        return i;
    return 0;

from math import *;


from collections import *;


def info():
    """
    Description:

        Functions in this Library:
        - Multimap (unique to fastcp package)
        - defaultdict
        - OrderedDict
        - namedtuple
        - deque
        - ChainMap
        - Counter
        - UserDict
        - UserList
        - UserString
    """
    print("""
    Description:

        Functions in this Library:
        - Multimap (unique to fastcp package)
        - defaultdict
        - OrderedDict
        - namedtuple
        - deque
        - ChainMap
        - Counter
        - UserDict
        - UserList
        - UserString
    """)
    
    
def Multimap(default):
    """
    This Function returns a Multimap (i.e Multi-DefaultDict) 
    
    Args:
        default (Any): the default value of multi-map will depend on the Argument passed
        0 -> Default value is Int
        [] -> Default value is List
        () -> Default value is Tuple
        {} -> Default value is Dict 
        0.0 -> Default value is Float
        True -> Default value is Bool
        etc ...
    Returns:
        defaultdict : returns a Multi-DefaultDict with default value as passed as Argument
    Example:
        >>> dictionary = Multimap(1) # takes int as default value (any int can be passed as argument)
        >>> dictionary[1][0] 
        0
        >>> dictionary = Multimap([])
        >>> dictionary[0][0]
        []
    """
    return defaultdict(lambda: defaultdict(type(default)));




from easycp import __version__
from easycp import math
from easycp import strings
from easycp import arrays
__version__ = "1.0.0"

def test_math():
    assert 3.0 == math.Cbrt(27);
    
def test_strings():
    assert 5 == strings.VowelCount("aeiou");

def test_arrays():
    assert 5 == arrays.Length([1,2,3,4,5]);


"""
    This library contains the following functions:
    - SortDict
    - Sort
    - SortDictValues
"""


def SortDict(dictionary, boolean = False) -> dict:
    """
    A Function that returns a Sorted Dictionary
    Args:
        dictionary (dict): Input Dictionary to Sort
        boolean (bool): True for Decreasing Sort, False for Increasing Sort
                        False by Default
    Returns:
        dict : returns a Sorted Dictionary
        Complexity: O(N LogN)
    Example:
        >>> dict = {10: 1, 8: 2, 1: 3, 4: 4}
        >>> print(SortDict(dict))
        {1: 3, 4: 4, 8: 2, 10: 1}
        >>> print(SortDict(dict, True))
        {10: 1, 8: 2, 4: 4, 1: 3}
    """
    newdict = {}
    for key in sorted(dictionary):
        newdict[key] = dictionary[key];
    return newdict;

def SortDictValues(dictionary, boolean = False):
    """
    A Function Sorts Dictionary based on Values of Dictionary
    Args:
        dictionary (dict): Input Dictionary to Sort
        boolean (bool): True for Decreasing Sort, False for Increasing Sort
                        False by Default
    Returns:
        dict : returns a Sorted Dictionary based on Values of Dictionary
        Complexity: O(N LogN)
    Example:
        >>> dict = {10: 1, 8: 2, 1: 3, 4: 4}
        >>> print(SortDict(dict))
        {10: 1, 8: 2, 1: 3, 4: 4}
        >>> print(SortDict(dict, True))
        {4: 4, 1: 3, 8: 2, 10: 1}
    """
    array = list(dictionary.keys()); newdict = {};
    array.sort(key = lambda x: dictionary[x], reverse = boolean);
    for i in array:
        newdict[i] = dictionary[i];
    return newdict;


def Sort(array, boolean = False):
    """
        This Function returns a Sorted List at a Complexity of O(N)
        Restriction: Maximum size of element is 10**8;
        Error Handling: Returns Index Error if Maximum element in List is Greater than 10**8;
    Args:
        array (List): take a input list
        boolean (bool): True for Decreasing Sort, False for Increasing Sort
                        False is taken as default Value
    Returns:
        List: Returns a Sorted List
        Complexity: O(N)
        Example:
            >>> array = [9, 1, 2, -1, -9 , 0, -100000000, 100000000]
            >>> Sort(array)
            [-100000000, -9, -1, 0, 1, 2, 9, 100000000]
            >>> array = [100000001]
            >>> Sort(array)
            IndexError: highest element in list cannot exceed 10**8
    """
    default = 100000000;
    pos_sort = [0]*(default); neg_sort = [0]*(default);
    newarr = []; pos = 0; neg = 0; poscount = 0; negcount = 0;
    try:
        for i in array:
            if (i < 0):
                if (abs(i) ==  default):
                    negcount += 1;
                else:
                    neg_sort[abs(i)] += 1; neg += 1;
            else:
                if (i == default):
                    poscount += 1;
                else:
                    pos_sort[i] += 1; pos += 1;
        if (negcount):
            while(negcount > 0):
                newarr.append(-default); negcount -= 1;
        if (neg):
            for i in range(default):
                if (neg_sort[i]):
                    while(neg_sort[i]):
                        newarr.append(-i); neg_sort[i] -= 1;
        if (pos):
            for i in range(default):
                if (pos_sort[i]):
                    while(pos_sort[i]):
                        newarr.append(i); pos_sort[i] -= 1;
        if (poscount):
            while(poscount > 0):
                newarr.append(default); poscount -= 1;
        del neg_sort; del pos_sort; del neg; del pos; del default;
        if (boolean): newarr.reverse();
        return newarr;          
    except IndexError:
        print("IndexError: highest element in list cannot exceed 10**8")
        return array;



"""
    Format: 
    
    class TreeNode:
        def __init__(self, val, left = None, right = None) -> None:
            self.val = val
            self.left = left
            self.right = right;
        
    Example: 
        root = TreeNode(10);
        root.left = TreeNode(5);
        root.right = TreeNode(20);
"""
#from collections import deque as _deque;

class TreeNode:
    def __init__(self, val, left = None, right = None) -> None:
        self.val = val
        self.left = left
        self.right = right;
        
def Create(value):
    """
    Args:
        value (int): value to insert into the TreeNode
    Returns:
        TreeNode: returns a node with value
        Complexity: O(1)
    
    Example: 
        root = create(10);
        root.left = trees.create(20); 
        root.right = trees.create(30);
    """
    node = TreeNode(value)
    return node
    
def Inorder(root):
    """
    Args:
        root (TreeNode): root node of Tree
    Returns:
        List: returns a List containing inorder traversal of nodes
        Complexity: O(Log N)
    
    Example: 
        arr = trees.inorder(root);
    """
    arr = []
    def traverse(root, arr):
        if (root == None): return;
        traverse(root.left, arr);
        arr.append(root.val);
        traverse(root.right, arr);
    traverse(root, arr);
    return arr;

def Preorder(root):
    """
    Args:
        root (TreeNode): root node of Tree
    Returns:
        List: returns a List containing preorder traversal of nodes
        Complexity: O(Log N)
        
    Example: 
        arr = trees.preorder(root);
    """
    arr = []
    def traverse(root, arr):
        if (root == None): return;
        arr.append(root.val);
        traverse(root.left, arr);
        traverse(root.right, arr);
    traverse(root, arr);
    return arr;

def Postorder(root):
    """
    Args:
        root (TreeNode): root node of Tree
    Returns:
        List: returns a List containing postorder traversal of nodes
        Complexity: O(Log N)
    
    Example: 
        arr = trees.postorder(root);
    """
    arr = []
    def traverse(root, arr):
        if (root == None): return;
        traverse(root.left, arr);
        traverse(root.right, arr);
        arr.append(root.val);
    traverse(root, arr);
    return arr;

def Levelorder(root):
    """
    Generates Level Order Traversal of Tree
    Args:
        root (TreeNode): Root node of tree
    Returns:
        List[List]: returns a 2D list containing nodes in level order traversal
        Complexity: O(N*N)
    
    Example: 
        arr = trees.levelorder(root);
    """
    arr = [];
    if (root):
        queue = _deque([root]); n = 1;
        while(n):
            vec = []; temp = None;
            for i in range(n):
                temp = queue.popleft();
                vec.append(temp.val);
                if (temp.left): queue.append(temp.left);
                if (temp.right): queue.append(temp.right);
            arr.append(vec); n = len(queue);
    del queue; del vec;
    return arr;

def NodeSum(root):
    """
    Returns the Sum of all the nodes in the Tree
    Args:
        root (TreeNode): Root node of tree
    Returns:
        Int : if tree is empty returns 0 else retuns the sum of all the nodes in the tree
        Complexity: O(N Log N)
    
    Example: 
        arr = trees.levelorder(root);
    """
    if root == None: return 0;
    return sum(Inorder(root));

def Search(root, key) -> bool:
    """
    A function that searches the tree for the given node
    Args:
        root (TreeNode): root node of tree to Search
        key (any): value/node to search for

    Returns:
        bool: returns True if node is found else returns False
    """
    def traverse(root, key):
        if root == None: return
        if root.val == key: return True;
        a = traverse(root.left, key)
        b = traverse(root.right, key);
        return a or b
    if traverse(root, key): return True;
    return False;





from collections import Counter as _Counter
from itertools import combinations as _combinations;
from itertools import permutations as _permutations;


def Freq(string):
    """A functions to get frequency of each character of string
    Args:
        string (str): _input string to get frequency
    Returns:
        dict: returns the dict of frequency of each character in string
        Complexity: O(N)
    """
    string = [i for i in string]
    return dict(_Counter(string));

def VowelCount(string):
    """A functions to count number of vowels in string
    Args:
        string (str): input string to get count of vowels
    Returns:
        int: returns the count of vowels in string
        Complexity: O(N)
    """
    vowel = ['a', 'e', 'i', 'o', 'u'];
    string = "".join([i for i in string]);
    string = string.lower(); count = 0;
    for i in string:
        if (i in vowel): count += 1;
    del vowel;
    return count;

def Substr(string):
    """A functions to get substrings of string
    Args:
        string (str): input string to get substring
    Returns:
        List: returns an List containing all substrings
        Complexity: O(N*N)
    """
    string = "".join([i for i in string]); substr = []; n = len(string);
    for i in range(n):
        ref = ''
        for j in range(i, n):
            ref += string[j];
            substr.append(ref);
    del ref;
    return substr;

def Subseq(string):
    """A function to get all subsequences of string

    Args:
        string (str): A string to get subsequences

    Returns:
        List: retuns a List containing all the subsequences of string
        Complexity: O(2**N)
    """
    string = "".join([str(i) for i in string])
    n = len(string); seq = []; ref = []
    def recur(string, n, i, seq, ref):
        if (i >= n): 
            seq.append("".join(ref)); return;
        ref.append(string[i]);
        recur(string, n, i+1, seq, ref);
        ref.pop();
        recur(string, n, i+1, seq, ref);
    recur(string, n, 0, seq, ref); del ref;
    return seq;

        
    
        